%% Directivity plot created out of two measurement campaigns
% Bieke von den Hoff
% PhD student ANCE - Aerospace Engineering - TU Delft 
% March 2021

clear all
close all
clc

%% Select for which mic want to plot the data
mic = 1;
dt_step = 0.125; % OSPL OASPL time step

%% Read basic data
% add path to the folder where all data is stored

% Note: MC1 - L3 = 90-180 deg, L4 = 0-90 deg, BUT MC2 - L2 = 90-180 deg, L3 = 0-90 deg
%% GPS data read out
GPS_settings1 = readtable('MC1_taxibot_GPS_data.xlsx','Sheet','Selectie tijden GPS data');
GPS_settings1(any(isnan(GPS_settings1{:,10}),2),:) = []; % delete rows where there is no offset data
GPS_taxibot1 = readtable('MC1_taxibot_GPS_data.xlsx','Sheet','GPS data taxibot');

GPS_settings2 = readtable('MC2_taxibot_GPS_data.xlsx','Sheet','Selectie tijden GPS data');
GPS_settings2(any(isnan(GPS_settings2{:,10}),2),:) = []; % delete rows where there is no offset data 
GPS_taxibot2 = readtable('MC2_taxibot_GPS_data.xlsx','Sheet','GPS data taxibot');

GPS_settings_comb = [GPS_settings1(:,1:13); GPS_settings2(:,1:13)]; % paste arrays under each other to ease the computations
GPS_taxibot_comb = [GPS_taxibot1(:,[1,2,3,7,11]); GPS_taxibot2(:,[1,2,3,7,11])];

%% Extract the GPS data 
for index_meas = 1:size(GPS_settings_comb,1) 
    clear t_old t_new OSPL_f OSPL_t OASPL t_mid lon_interp lat_interp d_interp OSPL_f_average data video t_steps PSD PSD_cal PSD_cor SPL SPL_cor d_lat d_lon OSPL_f OSPL_f_cal OSPL_t A_cor alpha OSPL_f_cor
    
    data.dir = GPS_settings_comb.FileNameNoiseInspector{index_meas};
    data.start_time = 1;
    data.end_time = 10;
    Read_data_CAE;
    
    lat_array = GPS_settings_comb.LatitudeArray(index_meas);
    lon_array = GPS_settings_comb.LongitudeArray(index_meas);
    meteo.temp = GPS_settings_comb.Temp(index_meas); 
    meteo.pres = GPS_settings_comb.Pres(index_meas); 
    meteo.humid = GPS_settings_comb.RH(index_meas);
    
    % GPS data improved - first smoothed then interpolated, lat and lon separately versus time
    start_time = datestr(addtodate(datenum(start_time,'yyyy-mm-dd HH:MM:SS'),GPS_settings_comb.time_offset(index_meas),'second'),'yyyy-mm-dd HH:MM:SS');

    end_time = GPS_settings_comb.EndTime_hh_mm_ss_{index_meas};
    end_time = datestr(addtodate(datenum(end_time,'yyyy-mm-dd HH:MM:SS'),GPS_settings_comb.time_offset(index_meas),'second'),'yyyy-mm-dd HH:MM:SS');

    % Find the GPS data for the measurement
    start_GPS = find(strcmp(GPS_taxibot_comb.ts,start_time));
    while isempty(start_GPS) == 1 % this means that the selected time from NoiseInspector does not have a matching point in the GPS data due to low sample rate
        start_time(end-7:end) = datestr(addtodate(datenum(start_time(end-7:end),'HH:MM:SS'),-1,'second'),'HH:MM:SS'); % start time taken one second earlier, work around with num2str/str2num to correct in case it ends at 0
        start_GPS = find(strcmp(GPS_taxibot_comb.ts,start_time));
    end

    end_GPS = find(strcmp(GPS_taxibot_comb.ts,end_time));
    while isempty(end_GPS) == 1
        end_time(end-7:end) = datestr(addtodate(datenum(end_time(end-7:end),'HH:MM:SS'),1,'second'),'HH:MM:SS'); 
        end_GPS = find(strcmp(GPS_taxibot_comb.ts,end_time));
    end

    lat_taxibot = GPS_taxibot_comb.lat(start_GPS:end_GPS);
    lon_taxibot = GPS_taxibot_comb.lon(start_GPS:end_GPS);
    time_stamp_GPS = GPS_taxibot_comb.interval(start_GPS:end_GPS);

    % Create old time vector for GPS data - interval is not always the same (varies between 1, 2, 3 sec)
    t_old(1) = str2num(GPS_settings_comb.SelectionStart_s_{index_meas}(end-1:end));
    for l = 2:length(time_stamp_GPS)
        t_old(l) = t_old(l-1)+time_stamp_GPS(l);
    end

    % Smooth the lat and lon data
    lon_smooth = smooth(t_old,lon_taxibot);
    lat_smooth = smooth(t_old,lat_taxibot);    

    info.pref = 2e-5;
    t_steps = t_old(1):dt_step:t_old(end);
    t_mid = t_steps(1:end-1)+dt_step/2;
    
    % Interpolate GPS data at time stamp of OSPL/OASPL
    lon_interp = interp1(t_old,lon_smooth,t_mid);
    lat_interp = interp1(t_old,lat_smooth,t_mid);

    for k = 1:length(t_steps)-1
        % Only look at the latitude or longitude distance between the array and the taxibot.
        % Enforced by setting the latitude or longitude to that of the array and varying the other
        d_lat(k) = lldistkm([lat_array lon_array],[lat_interp(k) lon_array])*1000; 
        d_lon(k) = lldistkm([lat_array lon_array],[lat_array lon_interp(k)])*1000;
        
        % Haversine formule calculates distance, not how they are located with respect to each other so correct that
        if lon_interp(k)<lon_array
            d_lon(k) = -d_lon(k);
        end
        if lat_interp(k)<lat_array
            d_lat(k) = -d_lat(k);
        end
    end
    
    if index_meas == 6 || index_meas == 7 || index_meas == 8 || index_meas == 16 || index_meas == 17 || index_meas == 18
        fit_lonlat = polyfit(d_lat,d_lon,10);
        d_lon = polyval(fit_lonlat,d_lat);
    end
    if index_meas == 9 || index_meas == 10 || index_meas == 11 || index_meas == 19 || index_meas == 20 || index_meas == 21
        fit_lonlat = polyfit(d_lon,d_lat,5);
        d_lat = polyval(fit_lonlat,d_lon);
    end
    
    d_interp = sqrt(d_lat.^2+d_lon.^2);
    
    figure(500)
    hold on
    plot3(d_lon,d_lat,t_mid)
end

%% Compute the OSPL and OASPL directionality plots
   
for k = 1:length(t_steps)-1  % Loop over the time steps  
    clear A_cor data.selected_data data.selected_time PSD SPL PSD_cal SPL_cal alpha
    
    data.start_time = t_steps(k);
    data.end_time = t_steps(k+1);

    %% Data selection and calibration
    data.start_sample = floor(data.start_time*info.fs);
    if data.start_sample == 0
        data.start_sample = 1;
    end
    data.end_sample = ceil(data.end_time*info.fs);
    data.selected_data = data.full_data(:,data.start_sample:data.end_sample);
    data.selected_time = data.full_time(data.start_sample:data.end_sample);

    % Data calibration correction in frequency domain
    for i = 1:length(config.x)
        [X_PSD,f_pwelch] = pwelch(data.selected_data(i,:),hann(block_size),block_overlap,zeropad*block_size,info.fs); % X_PSD = (abs(Xdft)).^2;
        PSD(i,:) = X_PSD;
    end

    df = f_pwelch(2)-f_pwelch(1);
    SPL = 10*log10(df*PSD/(info.pref^2));
    
    % Remove data below 20 Hz and above 20 kHz, because the correction is not accurate there
    I_f_sel = find(f_pwelch>20 & f_pwelch<20e3);
    f_pwelch = f_pwelch(I_f_sel);
    PSD = PSD(:,I_f_sel);
    SPL = SPL(:,I_f_sel);

     % Get positions in freq (same as coar_dB or cor_factor) where f=freq, to apply correction at different settings of fft 
    [~,matching_freqvec] = intersect(round(freq_cal),round(f_pwelch)); %due to small differences in decimals, need to round off to match all frequencies properly

    % Divide by the factor, because the correction = CAE/REFERENCE, so to get back to REFERENCE, need to divide by correction
    PSD_cal(I_middle,:) = PSD(I_middle,:)./cal_mid_factorPSD(matching_freqvec)';
    PSD_cal(I_normal,:) = PSD(I_normal,:)./cal_out_factorPSD(matching_freqvec)';

    SPL_cal = 10*log10(df*PSD_cal/(info.pref^2)); % Also equals SPL-cor_dB
            
    %% OSPL, OASPL calculation
    OSPL_f(:,k) = 10*log10(df*sum(PSD,2)/(info.pref^2));     % from frequency domain
    OSPL_f_cal(:,k) = 10*log10(df*sum(PSD_cal,2)/(info.pref^2));
    OSPL_t(k) = 10*log10(var(data.selected_data(mic,:))/(info.pref^2));  % from time domain

    % A-weighting and atmospheric absorption correction
    for f = 1:length(f_pwelch)
        A_cor(f) = -145.528 + 98.262*log10(f_pwelch(f)) - 19.509*(log10(f_pwelch(f)))^2 + 0.975*(log10(f_pwelch(f)))^3;
        alpha(f) = 20/log(10)*attenuation(f_pwelch(f),meteo);
    end
    
    % Remove background levels, so that they are not blown up with the absorption correction
    SPL_cal(SPL_cal <= 30) = NaN; 

    % Apply all corrections to get A-weighted source levels (A-weighting, spherical spreading, atmospheric attenuation)
    SPL_A_cor = SPL_cal + A_cor + 20*log10(d_interp(k)) + alpha*d_interp(k);
    SPL_A_cor(isnan(SPL_A_cor)) = 0; % if a value equals 0, it has no effect on the OASPL. It would be like taking it out.
    SPL_cor = SPL_cal + 20*log10(d_interp(k)) + alpha*d_interp(k);
    SPL_cor(isnan(SPL_cor)) = 0;
    
    OSPL_f_cor(:,k) = 10*log10(sum(10.^(SPL_cor./10),2));
    OASPL(:,k) = 10*log10(sum(10.^(SPL_A_cor./10),2));
end
  
I_mic = [1:112]; 
I_broken = [];
for i = 1:112
    broken = find(abs(OASPL(1,:)-OASPL(i,:))>10);
    if isempty(broken) == 0 && (length(broken)/size(OASPL,2))>=0.25
        I_broken = [I_broken i];
    end
end

I_mic(I_broken) = [];
OASPL_average = 10*log10(1/length(I_mic)*sum(10.^(OASPL(I_mic,:)./10),1));
OSPL_f_average = 10*log10(1/length(I_mic)*sum(10.^(OSPL_f_cor(I_mic,:)./10),1));


%% Plot the directionality lines
% Depending on the measurement location, the observer angle can be computed from geometry or had to be extracted from the videos.
% In the latter case, a file with the extracted angle is loaded. Information on the procedure can be found in the paper.

if (contains(data.dir,'L4_R1')+contains(data.dir,'MC1')+contains(data.dir,'Corendon'))==3  
    figure(100) 
    load('angleL3.mat')
    ObsAngle = polyval(fitangle_L3,d_lon)/180*pi;
    polarplot(ObsAngle,OSPL_f_average,'k')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'k')
    hold on
elseif (contains(data.dir,'L3_R1')+contains(data.dir,'MC1')+contains(data.dir,'Corendon'))==3 
    load('angleL2.mat')
    ObsAngle = polyval(fitangle_L2,d_lat)/180*pi;
    figure(100) 
    polarplot(ObsAngle,OSPL_f_average,'k')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'k')
    hold on
elseif (contains(data.dir,'L2_R1')+contains(data.dir,'MC1')+contains(data.dir,'Corendon'))==3 
    figure(100)     
    ObsAngle = [asin(d_interp(i_min)./d_interp(1:i_min)) pi-asin(d_interp(i_min)./d_interp(i_min+1:end))];
    polarplot(ObsAngle,OSPL_f_average,'k')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'k')
    hold on
elseif (contains(data.dir,'L1_R1')+contains(data.dir,'MC1')+contains(data.dir,'Corendon'))==3 
    figure(100)     
    ObsAngle = [asin(d_interp(i_min)./d_interp(1:i_min)) pi-asin(d_interp(i_min)./d_interp(i_min+1:end))];
    polarplot(ObsAngle,OSPL_f_average,'k')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'k')
    hold on
   
elseif (contains(data.dir,'L3_R1')+contains(data.dir,'MC2')+contains(data.dir,'Corendon'))==3 
    figure(100) 
    load('angleL3.mat')
    ObsAngle = polyval(fitangle_L3,d_lon)/180*pi;
    polarplot(ObsAngle,OSPL_f_average,'c')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'c')
    hold on
elseif (contains(data.dir,'L2_R1')+contains(data.dir,'MC2')+contains(data.dir,'Corendon'))==3  
    figure(100) 

    load('angleL2.mat')
    ObsAngle = polyval(fitangle_L2,d_lat)/180*pi;
    polarplot(ObsAngle,OSPL_f_average,'c')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'c')
    hold on
elseif (contains(data.dir,'L1_R1')+contains(data.dir,'MC2')+contains(data.dir,'Corendon'))==3  
    figure(100) 
    ObsAngle = [asin(d_interp(i_min)./d_interp(1:i_min)) pi-asin(d_interp(i_min)./d_interp(i_min+1:end))];
    polarplot(ObsAngle,OSPL_f_average,'c')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'c')
    hold on
elseif (contains(data.dir,'L4_R2')+contains(data.dir,'MC2')+contains(data.dir,'Corendon'))==3  
    figure(100) 
    ObsAngle = atan(-d_lat./d_lon)+0.5*pi;          
    polarplot(ObsAngle,OSPL_f_average,'c')
    hold on

    figure(101)
    polarplot(ObsAngle,OASPL_average,'c')
    hold on
    
end    

  
%% Create consistent layout    
figure(100)
hold on
title('Directionality plot - source levels - angle from GPS coordinates')
h1 = get(gcf,'Children');
set(h1,'FontSize',20)
ax = gca;
ax.RAxis.Label.String = 'OSPL';
% legend('OSPL_f MC1','OSPL_f MC2')
axis([0 180 90 130])
thetatickformat('degrees')
% legend(GPS_settings_comb.FileNameNoiseInspector)

figure(101)
hold on
title('Directionality plot - source levels - angle from GPS coordinates')
h1 = get(gcf,'Children');
set(h1,'FontSize',20)
ax = gca;
ax.RAxis.Label.String = 'OASPL';
% legend('OSPL_f MC1','OSPL_f MC2')
axis([0 180 90 130])
thetatickformat('degrees')
% legend(GPS_settings_comb.FileNameNoiseInspector)

% legend_list = findobj(gcf, 'Type', 'Legend');
% legend_list = legend_list.String;